from .entries.db.mysql_easy import MysqlEasyConnection
from .__init__ import BasicCheckUtil

class MysqlUtil:
    '''
    MysqlEasyConnection 简单操作工具类（建议使用）
    '''
    @staticmethod
    def connect(host, port, username, password, db=None):
        '''
        获取Mysql连接封装对象
        :param host: Mysql Host
        :param port: Mysql Port
        :param username: Mysql Username
        :param password: Mysql Password
        :param db: Mysql DataBase Name
        :return:MysqlEasyConnection
        '''
        return MysqlEasyConnection(host=host, port=port, user=username, password=password, database=db)

    @staticmethod
    def insert_dict(obj: MysqlEasyConnection, table: str, data: dict, mode='into'):
        '''
        插入数据
        :param obj: MysqlEasyConnection对象，通过MysqlUtil.connect获取
        :param table: 表名
        :param data: 字典结构数据,key 与 mysql 字段一一对应
        :param mode: 插入模式，包括 into(追加)，overwrite(覆盖)，ignore(忽略) 等
        :return: None.
        '''
        MysqlUtil.__check(obj)
        obj.insert_dict(table, data, mode)

    @staticmethod
    def insert_dicts(obj: MysqlEasyConnection, table: str, data: tuple or list, mode='into', every_length=200):
        '''
        批量插入数据
        :param obj: MysqlEasyConnection对象，通过MysqlUtil.connect获取
        :param table: 表名
        :param data: tuper或list类型，其中每个元素为dict类型
        :param mode: 插入模式，包括 into(追加)，overwrite(覆盖)，ignore(忽略) 等
        :param every_length: 分批插入的单批条数，例如：len(data)=1000，every_length=200，则会将数据分为5批，每批以200条数据插入
        :return: None.
        '''
        MysqlUtil.__check(obj)
        obj.insert_dicts(table, data, every_length, mode)

    @staticmethod
    def insert_dict_auto_create_table_and_column(obj: MysqlEasyConnection, table: str, data: dict, mode='into', primary_name='id'):
        '''
        插入单条字典类型，如果表不存在，则创建表，如果表中不包含某个列，将会在最后自动新增一列
        :param obj: MysqlEasyConnection对象，通过MysqlUtil.connect获取
        :param table: 表名
        :param data: dict类型
        :param mode: 插入模式，包括 into(追加)，overwrite(覆盖)，ignore(忽略) 等
        :param primary_name: 主键
        :return:
        '''
        MysqlUtil.__check(obj)
        obj.insert_dict_auto_add_table_and_column(table, data, mode, primary_name=primary_name)

    @staticmethod
    def update(obj: MysqlEasyConnection, sql, args=None):
        '''
        更新数据
        :param obj: MysqlEasyConnection对象，通过MysqlUtil.connect获取
        :param sql: 更新的sql
        :param args: 参数元组或列表
        :return: None.
        '''
        MysqlUtil.__check(obj)
        obj.update(sql, args)

    @staticmethod
    def query(obj: MysqlEasyConnection, sql, args=None, data_type='dict'):
        '''
        查询数据
        :param obj: MysqlEasyConnection对象，通过MysqlUtil.connect获取
        :param sql: 查询的sql
        :param args: 参数元组或列表
        :param data_type: 单元素类型，dict 或 tuper
        :return:tuper
        '''
        MysqlUtil.__check(obj)
        return obj.query(sql, args, data_type)

    @staticmethod
    def delete(obj: MysqlEasyConnection, sql, args=None):
        '''
        删除数据
        :param obj: MysqlEasyConnection对象，通过MysqlUtil.connect获取
        :param sql: 删除的sql
        :param args: 参数元组或列表
        :return: None.
        '''
        MysqlUtil.__check(obj)
        return obj.delete(sql, args)

    @staticmethod
    def __check(obj, *args, **kwargs):
        if BasicCheckUtil.is_none(obj):
            raise RuntimeError("MysqlEasyConnection对象不存在")



class BasicCheckUtil:
    '''
    todo 基本数据类型检测工具类
    '''
    @classmethod
    def is_empty(cls, param):
        '''
        todo 判断对象是否为空，添加对基本数据类型的检测
        :param param: 参数
        :return:
        '''
        if cls.is_none(param):
            return True
        if type(param) in (int,float) and param == 0:
            return True
        if type(param) in (str,) and param == '':
            return True
        if type(param) in (tuple, list, dict) and len(param) == 0:
            return True
        if type(param) in (bool,) and param == False:
            return True
        return False

    @classmethod
    def non_empty(cls, param):
        '''
        todo 判断对象是否非空，添加对基本数据类型的检测
        :param param: 参数
        :return:
        '''
        return not cls.is_empty(param)

    @classmethod
    def is_none(cls, param):
        '''
        todo 判断对象是否为空
        :param param: 参数
        :return:
        '''
        if param is None:
            return True
        return False

    @classmethod
    def non_none(cls, param):
        '''
        todo 判断对象是否非空
        :param param: 参数
        :return:
        '''
        return not cls.is_none(param)

    @classmethod
    def equels(cls, a, b):
        if type(a) == type(b):
            if (type(a) in (int, float, str, bool)):
                if a == b:
                    return True
                return False
    @classmethod
    def contains(cls, a, b):
        '''
        a contains b ?
        :param a:
        :param b:
        :return:
        '''
        return b in a



def send_email_old(host: str, port: int, send_email: str, send_user: str, send_pass: str, receive:list,
           title: str, message: str, message_type='plain', encoding='utf-8', is_ssl=True, **kwargs):
    '''
    SMTP 发送邮件服务
    :param host: smtp服务器地址
    :param port: smtp服务器端口
    :param send_email: 发送人邮箱
    :param send_user: 发送人名称
    :param send_pass: 发送人邮箱的发送密码
    :param receive: 接收人邮箱列表
    :param title: 标题
    :param message: 正文
    :param message_type: 消息类型，plain, html
    :param encoding: 内容编码
    :param is_ssl: 是否为SSL连接的服务器
    :param kwargs: 扩展字段
    :return: {"status": "success"} 或 "status": "fail"}
    '''
    if BasicCheckUtil.equels(message_type, 'plain'):
        from smtplib import SMTP_SSL
        import smtplib
        from email.mime.text import MIMEText
        from email.mime.multipart import MIMEMultipart
        from email.header import Header
        if not is_ssl:
            smtp_obj =smtplib.SMTP(host, port)
        else:
            smtp_obj = SMTP_SSL(host, port)
        smtp_obj.login(send_email, send_pass)
        message_obj = MIMEText(message, message_type, encoding)
        mime_message_obj = MIMEMultipart('related')
        mime_message_obj.attach(message_obj)
        mime_message_obj['Subject'] = title
        mime_message_obj['From'] = Header(send_user, encoding)
        for receive_mail in receive:
            mime_message_obj['To'] = Header(receive_mail, encoding)
        smtp_obj.sendmail(send_email, receive, mime_message_obj.as_string())
        return {'status': 'success'}

def send_mail(host: str, port: int, send_email: str, send_pass: str, receive:list, title: str, message: str, is_ssl=True):
    import zmail
    obj = zmail.server(send_email, send_pass, host, port, is_ssl)
    send_message = {
        'subject': title,
        'content_text': message,
    }
    obj.send_mail(receive, send_message)


import os
import xlrd
from .__init__ import *

class XlsxFileEasyEntry:
    '''
    todo 基于 xlrd 模块的简单封装，简化 xlrd 操作
    '''
    __file_path = None
    __book = None # 工作簿对象
    __sheet_names = []
    __sheet = {} # sheet对象存储
    __sheet_dis = {} # sheet解析后存储
    __rows = {} # 行数存储字典
    __cols = {} # 列数存储字典
    __default_sheel_key = None
    def __init__(self, path: str, default_sheet=None):
        self.__check(path)
        self.__file_path = os.path.abspath(path)
        self.__init_book()
        if BasicCheckUtil.non_none(default_sheet):
            if type(default_sheet) in (str, int):
                self.__init_sheet(sheet=default_sheet)
            elif type(default_sheet) in (list, tuple):
                for st in default_sheet:
                    self.__init_sheet(sheet=st)


    def find(self, sheet=None, start_row=None, start_col=None, end_row=None, end_col=None):
        '''
        基于位置查找xlsx文件中的内容
        支持单行查找：start_row=number, 单列查找：start_col=number，返回list
        支持指定元素查找：start_row=number1, start_col=number2，返回str
        支持范围查找：start_row=number1, start_col=number2, end_row=number3, end_col=number4, 返回list[list]
        其中参数要求：
        start_row 与 start_col 不能同时为空
        end_row 不为空时 start_row 也不能为空
        end_col 不为空时 start_col 也不能为空
        :param sheet: sheet名，值为None时使用默认sheet名
        :param start_row: 开始行数，0开始
        :param start_col: 开始列数，0开始
        :param end_row: 结束行数，1开始
        :param end_col: 结束列数，1开始
        :return: list 或 str
        '''
        if BasicCheckUtil.is_empty(sheet):
            self.__check_sheet(self.__default_sheel_key)
            local_sheet_key = self.__default_sheel_key
        else:
            self.__init_sheet(sheet=sheet)
            self.__check_sheet(sheet)
            local_sheet_key = sheet
        result_list = []
        if BasicCheckUtil.is_none(start_row) and BasicCheckUtil.is_none(start_col):
            raise ValueError('start_row 与 start_col 不能同时为空')
        if BasicCheckUtil.is_none(start_col):
            if BasicCheckUtil.is_none(end_row):
                result_list = self.__rows[local_sheet_key][start_row]
            else:
                for index in range(start_row, end_row):
                    result_list.append(self.__rows[local_sheet_key][index])
        elif BasicCheckUtil.is_none(start_row):
            if BasicCheckUtil.is_none(end_col):
                result_list = self.__cols[local_sheet_key][start_col]
            else:
                for index in range(start_col, end_col):
                    result_list.append(self.__cols[local_sheet_key][index])
        else:
            if not ((BasicCheckUtil.is_none(end_col) and BasicCheckUtil.is_none(end_row))
                    or (BasicCheckUtil.non_none(end_col) and BasicCheckUtil.non_none(end_row))):
                raise ValueError('当start_row 与 start_col 同时存在值时，end_row 与 end_col 必须同时存在值或同时为 None')
            if BasicCheckUtil.is_none(end_col) and BasicCheckUtil.is_none(end_row):
                result_list = self.__rows[local_sheet_key][start_row][start_col]
            else:
                for index in range(start_row, end_row):
                    result_list.append(self.__rows[local_sheet_key][index][start_col:end_col])
        return result_list

    def rows(self, sheet=None):
        '''
        获取行数
        :param sheet: 对应的xlsx的sheet名，当为None时使用默认的sheet名
        :return:
        '''
        if BasicCheckUtil.is_empty(sheet):
            self.__check_sheet(self.__default_sheel_key)
            return len(self.__rows[self.__default_sheel_key])
        else:
            self.__init_sheet(sheet=sheet)
            self.__check_sheet(sheet)
            return len(self.__rows[sheet])

    def cols(self, sheet=None):
        '''
        获取列数
        :param sheet: 对应的xlsx的sheet名，当为None时使用默认的sheet名
        :return:
        '''
        if BasicCheckUtil.is_empty(sheet):
            self.__check_sheet(self.__default_sheel_key)
            return len(self.__cols[self.__default_sheel_key])
        else:
            self.__init_sheet(sheet=sheet)
            self.__check_sheet(sheet)
            return len(self.__cols[sheet])

    def sheets(self):
        self.__init_book()
        return self.__sheet_names

    def __init_sheet(self, **kwargs):
        for key, value in kwargs.items():
            value_str = str(value)
            if BasicCheckUtil.equels(key, 'sheet') and BasicCheckUtil.is_none(self.__sheet.get(value_str)):
                if type(value) in (int,):
                    self.__sheet[value_str] = self.__book.sheet_by_index(value)
                elif type(value) in (str,):
                    self.__sheet[value_str] = self.__book.sheet_by_name(value)
                if BasicCheckUtil.is_none(self.__default_sheel_key):
                    self.__default_sheel_key = value_str
                self.__rows[value_str] = []
                self.__cols[value_str] = []
                for row_index in range(self.__sheet[value_str].nrows):
                    self.__rows[value_str].append(self.__sheet[value_str].row_values(row_index))
                for col_index in range(self.__sheet[value_str].ncols):
                    self.__cols[value_str].append(self.__sheet[value_str].col_values(col_index))

    def __init_book(self):
        if BasicCheckUtil.is_none(self.__book):
            self.__book = xlrd.open_workbook(self.__file_path)
            self.__sheet_names = self.__book.sheet_names()

    def __check(self, path):
        if BasicCheckUtil.is_empty(path):
            raise ValueError('文件路径不能为空')

    def __check_sheet(self, sheet_key):
        if BasicCheckUtil.is_none(self.__sheet.get(sheet_key)):
            raise ValueError('没有指定的sheet对象')


try:
    from setuptools import setup
except:
    from distutils.core import setup
import setuptools

setup(
    name='pyeasytd',
    author='dragons',
    version='0.0.8',
    description='使Python开发变的更简单',
    long_description='Python 简化处理工具，轻松、快捷编码，使Python开发变的更简单。',
    author_email='521274311@qq.com',
    url='https://gitee.com/kingons/pyeasytd',
    packages=setuptools.find_packages(),
    install_requires= [
        # 'fake-useragent>=0.1.11',
        'PyMySQL>=0.9.3',
        'xlrd>=1.2.0',
        'zmail>=0.2.8',
        # 'numpy>=1.17.4',
        # 'moviepy>=1.0.1',
        # 'Pillow>=6.2.1',
        'requests>=2.22.0',
        # 'urllib3>=1.25.7',
        # 'bs4>=0.0.1',
    ],
    classifiers=[
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3.7',
        'Topic :: Software Development :: Libraries',
    ],
    zip_safe=True,
)